﻿using System;
using System.Windows;
using System.Windows.Input;
using Telerik.Windows;
using RadRoutedEvent = Telerik.Windows.RoutedEvent;

namespace BugTrackerDragDrop.Controls
{
	/// <summary>
	/// A helper class that adds a ClickEvent to any framework element that does not handle the 
	/// mouse events.
	/// </summary>
	public static class ClickHelper
	{
		public static readonly RadRoutedEvent ClickEvent = EventManager.RegisterRoutedEvent("Click", RoutingStrategy.Bubble, typeof(ClickEventHnadler), typeof(ClickHelper));

		public static readonly DependencyProperty EnableRoutedClickProperty =
			DependencyProperty.RegisterAttached("EnableRoutedClick",
			typeof(bool),
			typeof(ClickHelper), 
			new Telerik.Windows.PropertyMetadata(OnEnableRoutedClickChanged));

		private static readonly DependencyProperty ClickInfoProperty =
			DependencyProperty.RegisterAttached("ClickInfo", typeof(ClickInfo), typeof(ClickHelper), null);

		private static readonly TimeSpan MultiClickSpan = TimeSpan.FromSeconds(0.5);

		public static bool GetEnableRoutedClick(DependencyObject obj)
		{
			return (bool)obj.GetValue(EnableRoutedClickProperty);
		}

		public static void SetEnableRoutedClick(DependencyObject obj, bool value)
		{
			obj.SetValue(EnableRoutedClickProperty, value);
		}

		internal static ClickInfo GetClickInfo(DependencyObject obj)
		{
			return (ClickInfo)obj.GetValue(ClickInfoProperty);
		}

		internal static void SetClickInfo(DependencyObject obj, ClickInfo value)
		{
			obj.SetValue(ClickInfoProperty, value);
		}

		private static void OnEnableRoutedClickChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
		{
			var newValue = Convert.ToBoolean(e.NewValue);
			var element = sender as FrameworkElement;

			if (element != null)
			{
				if (newValue)
				{
					element.MouseLeftButtonDown += new MouseButtonEventHandler(OnElementMouseLeftButtonDown);
					element.MouseLeftButtonUp += new MouseButtonEventHandler(OnElementMouseLeftButtonUp);
				}
			}
		}		

		private static void OnElementMouseLeftButtonUp(object sender, MouseButtonEventArgs e)
		{
			var element = sender as FrameworkElement;
			var info = GetClickInfo(element);
			if (info == null)
			{
				info = new ClickInfo();
			}
			var clickPoint = e.GetPosition(element);

			if (info.LastClickPoint != clickPoint)
			{
				info = null;
			}
			else
			{
				var elasped = DateTime.Now - info.LastClickTime;
				if (elasped > MultiClickSpan)
				{
					info.ClickCount = 0;
				}
				info.ClickCount++;
				element.RaiseEvent(new ClickEventArgs(ClickEvent, element, info.ClickCount));
				info.LastClickTime = DateTime.Now;
			}

			SetClickInfo(element, info);
		}

		private static void OnElementMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
		{
			var element = sender as FrameworkElement;
			var info = GetClickInfo(element);
			if (info == null)
			{
				info = new ClickInfo();
			}
			var clickPoint = e.GetPosition(element);

			if (info.LastClickPoint != clickPoint)
			{
				info.ClickCount = 0;
				info.LastClickPoint = clickPoint;
			}

			SetClickInfo(element, info);
		}

		internal class ClickInfo
		{
			internal int ClickCount
			{
				get;
				set;
			}
			internal DateTime LastClickTime
			{
				get;
				set;
			}
			internal Point LastClickPoint
			{
				get;
				set;
			}
		}
	}
}
