﻿using System.Windows;
using Telerik.Windows;
using Telerik.Windows.Controls;
using RadItemsControl = Telerik.Windows.Controls.ItemsControl;

namespace BugTrackerDragDrop.Controls
{
	/// <summary>
	/// A Simple ItemsControl that generates Expanders for its items.
	/// </summary>
    public class MultiExpander : RadItemsControl
    {



		public DataTemplate ContentTemplate
		{
			get { return (DataTemplate)GetValue(ContentTemplateProperty); }
			set { SetValue(ContentTemplateProperty, value); }
		}

		// Using a DependencyProperty as the backing store for ContentTemplate.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty ContentTemplateProperty =
			DependencyProperty.Register("ContentTemplate", typeof(DataTemplate), typeof(MultiExpander), null);



		public DataTemplateSelector ContentTemplateSelector
		{
			get { return (DataTemplateSelector)GetValue(ContentTemplateSelectorProperty); }
			set { SetValue(ContentTemplateSelectorProperty, value); }
		}

		// Using a DependencyProperty as the backing store for ContentTemplateSelector.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty ContentTemplateSelectorProperty =
			DependencyProperty.Register("ContentTemplateSelector", typeof(DataTemplateSelector), typeof(MultiExpander), null);


        

        public MultiExpander()
        {
            this.DefaultStyleKey = typeof(MultiExpander);
            this.AddHandler(RadExpander.ExpandedEvent, new RoutedEventHandler(this.OnItemExpand));
			this.AddHandler(RadExpander.CollapsedEvent, new RoutedEventHandler(this.OnItemCollapse)); 
        }

        public void OnItemExpand(object sender, RoutedEventArgs args)
        {
            var expnder = (args as RadRoutedEventArgs).OriginalSource as RadExpander;
            if (expnder != null && RadItemsControl.ItemsControlFromItemContainer(expnder) == this)
            {
                PanelBarPanel.SetDesiredHeight(expnder, new GridLength(1, GridUnitType.Star));
            }
        }

        public void OnItemCollapse(object sender, RoutedEventArgs args)
        {
            var expnder = (args as RadRoutedEventArgs).OriginalSource as RadExpander;
            if (expnder != null && RadItemsControl.ItemsControlFromItemContainer(expnder) == this)
            {
                PanelBarPanel.SetDesiredHeight(expnder, GridLength.Auto);
            }
        }

        protected override bool IsItemItsOwnContainerOverride(object item)
        {
            return item is RadExpander;
        }

        protected override DependencyObject GetContainerForItemOverride()
        {
            return new RadExpander();
        }

		protected override void PrepareContainerForItemOverride(DependencyObject element, object item)
		{
			base.PrepareContainerForItemOverride(element, item);

			var headeredContentControl = element as HeaderedContentControl;
			if (headeredContentControl.ContentTemplate == null)
			{
				headeredContentControl.ContentTemplate = this.ContentTemplate;
				if (this.ContentTemplate == null && this.ContentTemplateSelector != null)
				{
					headeredContentControl.ContentTemplate = this.ContentTemplateSelector.SelectTemplate(item, element);
				}
			}
		}
    }
}
