﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
// #IncludeNamespaces Note that you need to include these two namespaces if you
// wish to use the in code.
using Telerik.Windows;
using Telerik.Windows.Controls.DragDrop;

using BugTrackerDragDrop.Views;
using Telerik.Windows.Controls;
using BugTrackerDragDrop.ViewModels;
using System.Diagnostics;
using BugTrackerDragDrop.Controls;
using Telerik.Windows.Controls.GridView;

namespace BugTrackerDragDrop
{
	public partial class Page : UserControl
	{
		public Page()
		{
			InitializeComponent();

			ApplicationDragDrop.Initialize();

			// #RootHandlerTrue: Add a DragQuery handler, since it is added to the root visual
			// it will catch all DragQuery events. It will even listen for
			// handled events, because we have used the overloaded AddHnadler
			// method with a last argument that specifies whether we will
			// handle handed events.
			this.AddHandler(RadDragAndDropManager.DragQueryEvent,
				new EventHandler<DragDropQueryEventArgs>(AllowAllDragsEventHandler), true);

			this.AddHandler(RadDragAndDropManager.DropInfoEvent,
				new EventHandler<DragDropEventArgs>(ResetPayLoadDropInfo), true);

			// #HandleDoubleClick: This handler makes sure that whenever you
			// double click a bug item, you will open and view it.
			this.AddHandler(ClickHelper.ClickEvent, new ClickEventHnadler(OnClick));

			// #AddHandler: This is how you add a handler to a routed event. Routed
			// events differ from standard events because they travell up and down
			// the visual trying to find a handler that they can invoke.
			this.AddHandler(GridViewCell.MouseDoubleClickEvent, new RoutedEventHandler(OnGridViewCellDoubleClick));
		}

		private void OnGridViewCellDoubleClick(object sender, RoutedEventArgs e)
		{
			var routedArgs = e as RadRoutedEventArgs;
			var bug = (routedArgs.OriginalSource as FrameworkElement).DataContext as BugViewModel;

			if (bug != null)
			{
				bug.IsSelected = true;
				BugTrackerViewModel.Instance.SelectedBug = bug;
			}
		}

		private void OnClick(object sender, ClickEventArgs e)
		{
			var bug = (e.OriginalSource as FrameworkElement).DataContext as BugViewModel;

			// Notice that we can use this as single/double/,multiple click
			// event depending on the count:
			if (bug != null && e.ClickCount == 2)
			{
				bug.IsSelected = true;
				BugTrackerViewModel.Instance.SelectedBug = bug;
			}
		}

		private void AllowAllDragsEventHandler(object sender, DragDropQueryEventArgs e)
		{
			var payload = e.Options.Payload as DragDropViewModel;
			if (e.Options.Status == DragStatus.DragQuery && e.QueryResult != false)
			{
				if (payload != null)
				{
					// DD: Say YES to all drags:
					e.QueryResult = true;
					e.Handled = true;

					// DD: Show an arrow:
					// DD: We can give any element as an arrow, this happens to streth nicely as needed.
					e.Options.ArrowCue = RadDragAndDropManager.GenerateArrowCue();

					var template = DataTemplatesProvider.Instance.DragCueTemplate;

					e.Options.DragCue = new ContentControl()
					{
						ContentTemplate = template,
						Content = payload
					};
				}
			}

			if (e.Options.Payload != null && e.Options.Status == DragStatus.DropSourceQuery)
			{
				if (e.QueryResult == null)
				{
					e.QueryResult = true;
				}
			}
		}

		private void ResetPayLoadDropInfo(object sender, DragDropEventArgs e)
		{
			var payload = e.Options.Payload as DragDropViewModel;

			if (e.Options.Status == DragStatus.DropImpossible && payload != null)
			{
				// DD: Reset the DragDropViewModel:
				payload.IconPath = null;
				payload.Message = null;
				payload.ApprovedDrops = null;
			}
		}

	}
}
