﻿using System.Windows;
using BugTrackerDragDrop.ViewModels;

namespace BugTrackerDragDrop.Utils
{
    /// <summary>
    ///     Class that helps us during design-time.
    /// </summary>
    /// <remarks>
    ///     Developer and designers can work together only if they make the
    ///     effort to. By adding dummy data contexts where needed we (the
    ///     developers) aid the designers.
    ///     
    ///     Please note that the dummy context will only appear duging
    ///     design-time.
    ///     
    ///     Unfortunately the object that we give the attached property will
    ///     always be created this is why we make sure that actually only one
    ///     instance of the main BugTrackerViewModel
    ///     
    ///     Maybe it is a good idea to remove it from Utils. And then remove
    ///     utils.
    /// </remarks>
	public static class Design
	{
		public static readonly DependencyProperty DummyDataContextProperty =
			DependencyProperty.RegisterAttached("DummyDataContext", typeof(object), typeof(Design), new PropertyMetadata(OnDummyDataContextChanged));

		public static object GetDummyDataContext(DependencyObject obj)
		{
			return (object)obj.GetValue(DummyDataContextProperty);
		}

		public static void SetDummyDataContext(DependencyObject obj, object value)
		{
			obj.SetValue(DummyDataContextProperty, value);
		}

		private static void OnDummyDataContextChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
		{
			var visual = sender as FrameworkElement;
			if (BugTrackerViewModel.IsInDesignMode && visual != null)
			{
				RoutedEventHandler visual_Loaded = null;
				visual_Loaded = (s, a) =>
					{
						if (visual.DataContext == null)
						{
							var singleInstance = e.NewValue as ISingleInstance;
							visual.DataContext = singleInstance != null ? singleInstance.SingleInstance : e.NewValue;
						}
						visual.Loaded -= visual_Loaded;
				 };
				visual.Loaded += visual_Loaded;
			}
		}
	}
}
