﻿using System;
using System.Collections.Generic;
using Bindable.Linq;

namespace BugTrackerDragDrop.ViewModels
{
	public class TeamViewModel : ViewModelBase, IInfoProvider
	{
		private TeamViewModel parentTeam;
		private IEnumerable<object> children;
		private IEnumerable<TeamViewModel> teams;
		private IEnumerable<TeamMemberViewModel> members;
		private string name;
		private bool isCompared;

		/// <summary>
		/// Gets or sets the parent team of this team.
		/// </summary>
		public TeamViewModel ParentTeam
		{
			get
			{
				return this.parentTeam;
			}
			set
			{
				if (this.parentTeam != value)
				{
					this.parentTeam = value;
					OnPropertyChanged("ParentTeam");
				}
			}
		}

		/// <summary>
		/// Gets or sets the children (teams and people) of this team.
		/// </summary>
		public IEnumerable<object> Children
		{
			get
			{
				return this.children ?? (this.children =
					this.Teams.AsBindable<object>().Concat(this.Members.AsBindable<object>()));
			}
		}

		/// <summary>
		/// Gets or sets the teams part of this team.
		/// </summary>
		public IEnumerable<TeamViewModel> Teams
		{
			get
			{
				return this.teams ?? (this.teams = 
					BugTrackerViewModel.Instance.AllTeams.AsBindable().Where(team => team.ParentTeam == this));
				
				// .OrderByDescending(team => team.TeamName));
			}
		}

		/// <summary>
		/// Gets or sets the people in this team.
		/// </summary>
		public IEnumerable<TeamMemberViewModel> Members
		{
			get
			{
				return this.members ?? (this.members = 
					BugTrackerViewModel.Instance.AllTeamMembers.AsBindable().Where(member => member.Team == this));
				
				// .OrderByDescending(member => member.FullName));
			}
		}

		/// <summary>
		/// Gets or sets name of the team.
		/// </summary>
		public string TeamName
		{
			get
			{
				return this.name;
			}
			set
			{
				if (this.name != value)
				{
					this.name = value;
					OnPropertyChanged("TeamName");
				}
			}
		}

        /// <summary>
        ///     Gets or sets whether the item is actively compared.
        /// </summary>
        public bool IsCompared
        {
            get
            {
                return this.isCompared;
            }
            set
            {
                if (this.isCompared != value)
                {
                    this.isCompared = value;
                    OnPropertyChanged("IsCompared");
                }
            }
        }
    }
}
