﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Controls;
using System.Windows.Media;
using BugTrackerDragDrop.ViewModels;
using Telerik.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.DragDrop;
using Telerik.Windows.Controls.GridView;
using Telerik.Windows.Data;

namespace BugTrackerDragDrop.Views
{
	public class ApplicationDragDrop
	{
		private static bool isInitialized;

		/// <summary>
		/// This method iinitializes application-wide drag-drop rules. We
		/// register them with class event handlers and this way we do not have
		/// to ever know the instances of where this effect takes place.
		/// #Initialize.
		/// </summary>
		public static void Initialize()
		{
			if (isInitialized)
			{
				return;
			}
			isInitialized = true;

			EventManager.RegisterClassHandler(typeof(RadExpander),
				RadDragAndDropManager.DropQueryEvent,
				new EventHandler<DragDropQueryEventArgs>(OnExpanderDropQuery));

			EventManager.RegisterClassHandler(typeof(RadTreeViewItem),
				RadDragAndDropManager.DropQueryEvent,
				new EventHandler<DragDropQueryEventArgs>(OnTrewViewItemDropQuery));

			EventManager.RegisterClassHandler(typeof(RadTreeView),
				RadDragAndDropManager.DragQueryEvent,
				new EventHandler<DragDropQueryEventArgs>(OnTrewViewDragQuery));

			EventManager.RegisterClassHandler(typeof(System.Windows.Controls.ListBox),
				RadDragAndDropManager.DragQueryEvent,
				new EventHandler<DragDropQueryEventArgs>(OnListBoxDragQuery));

			EventManager.RegisterClassHandler(typeof(RadGridView),
				RadDragAndDropManager.DragQueryEvent,
				new EventHandler<DragDropQueryEventArgs>(OnGridViewDragQuery));

			EventManager.RegisterClassHandler(typeof(Control),
				RadDragAndDropManager.DropInfoEvent,
				new EventHandler<DragDropEventArgs>(OnControlDropInfo));

			EventManager.RegisterClassHandler(typeof(RadComboBox),
				RadDragAndDropManager.DropQueryEvent,
				new EventHandler<DragDropQueryEventArgs>(OnComboBoxDragQuery));

			EventManager.RegisterClassHandler(typeof(RadTabItem),
				RadDragAndDropManager.DropQueryEvent,
				new EventHandler<DragDropQueryEventArgs>(OnTabItemDropQuery));

			EventManager.RegisterClassHandler(typeof(GridViewRow),
				RadDragAndDropManager.DropQueryEvent,
				new EventHandler<DragDropQueryEventArgs>(OnGridViewRowDropQuery));

			EventManager.RegisterClassHandler(typeof(GridViewRow),
				RadDragAndDropManager.DropInfoEvent,
				new EventHandler<DragDropEventArgs>(OnGridViewRowDropInfo));

			EventManager.RegisterClassHandler(typeof(GridViewScrollViewer),
				RadDragAndDropManager.DropQueryEvent,
				new EventHandler<DragDropQueryEventArgs>(OnGridViewScrollViewerDropQuery),
				true);
		}

		private static void OnExpanderDropQuery(object sender, DragDropQueryEventArgs e)
		{
			// #ExpnaderExpand: We always want to expand the underlying expanders if they
			// are collapsed when the user drags over them.
			var expander = sender as RadExpander;
			if (expander != null && !expander.IsExpanded && e.Options.Status == DragStatus.DropDestinationQuery)
			{
				expander.IsExpanded = true;
			}
		}

		private static void OnTrewViewItemDropQuery(object sender, DragDropQueryEventArgs e)
		{
			// #TreeViewExpand: We always want to expand the underlying treeViewItems if they
			// are collapsed when the user drags over them.
			var treeViewItem = sender as RadTreeViewItem;
			if (treeViewItem != null && !treeViewItem.IsExpanded && e.Options.Status == DragStatus.DropDestinationQuery)
			{
				treeViewItem.IsExpanded = true;
			}
		}

		private static void OnComboBoxDragQuery(object sender, DragDropQueryEventArgs e)
		{
			//// #ComboExpand: The code bellow will automatically open
			//// ComboBoxes. Unfortunately there is a bug in Silverlight that
			//// makes changing the ZIndex of a Popup impossible. Therefore
			//// the code below will lead to undesired results when the Combo
			//// DropDown goes over the DragCue. 
			// var comboBox = sender as RadComboBox;
			// if (comboBox != null && !comboBox.IsDropDownOpen
			//    && e.Options.Status == DragStatus.DropDestinationQuery)
			// {
			//    comboBox.IsDropDownOpen = true;
			// }
		}

		private static void OnTrewViewDragQuery(object sender, DragDropQueryEventArgs e)
		{
			// #SelectedTreeItems: We do not mind dragging items from our treeView. In fact
			// we will include all selected items as dragged items, as long 
			// as the source item is selected.
			var treeView = sender as RadTreeView;
			var treeViewItem = e.Options.Source as RadTreeViewItem;

			if (treeView != null && treeViewItem != null)
			{
				if (e.Options.Status == DragStatus.DragQuery && treeViewItem.IsSelected)
				{
					var payload = new DragDropViewModel()
					{
						DraggedItems = treeView.SelectedItems.ToList()
					};
					e.Options.Payload = payload;

					e.Handled = true;
					e.QueryResult = true;
				}
			}
		}

		private static void OnListBoxDragQuery(object sender, DragDropQueryEventArgs e)
		{
			// #SelectedListBoxItems: We do not mind dragging items from our
			// listBoxes. In fact we will include all selected items as dragged
			// items, as long as the source item is selected.
			var listBox = sender as System.Windows.Controls.ListBox;
			var listBoxItem = e.Options.Source as System.Windows.Controls.ListBoxItem;

			if (listBox != null && listBoxItem != null)
			{
				if (e.Options.Status == DragStatus.DragQuery)
				{
					var payload = new DragDropViewModel()
					{
						DraggedItems = new List<object>() { listBox.SelectedItem }
					};
					e.Options.Payload = payload;

					e.Handled = true;
					e.QueryResult = true;
				}
			}
		}

		private static void OnGridViewDragQuery(object sender, DragDropQueryEventArgs e)
		{
			// #GridViewRowsDrag: We always allow dragging all selected GridViewRows:
			var gridView = sender as RadGridView;
			var gridRow = e.Options.Source as GridViewRow;

			if (gridView != null && gridRow.IsSelected)
			{
				if (e.Options.Status == DragStatus.DragQuery && gridRow.IsSelected)
				{
					var payload = new DragDropViewModel()
					{
						DraggedItems = gridView.SelectedItems.ToList()
					};
					e.Options.Payload = payload;

					e.Handled = true;
					e.QueryResult = true;
				}
			}
		}

		private static void OnTabItemDropQuery(object sender, DragDropQueryEventArgs e)
		{
			// #TabAutoSwitch: Here we make sure that TabItems are
			// automatically selected when we drag over them. This will work
			// both for the TabControl and RadPaneGroup
			var tabItem = sender as RadTabItem;
			if (tabItem != null && !tabItem.IsSelected
				&& e.Options.Status == DragStatus.DropDestinationQuery)
			{
				tabItem.IsSelected = true;
			}
		}

		private static void OnGridViewRowDropQuery(object sender, DragDropQueryEventArgs e)
		{
			// #GridViewMemberDrop: We want to allow TeamMembers to be dropped
			// on to bugs. This is not a very flexible approach, but it shows
			// how you can enable a business rule for a given type.
			var gridRow = sender as GridViewRow;
			var payload = e.Options.Payload as DragDropViewModel;

			if (gridRow != null
				&& e.Options.Status == DragStatus.DropDestinationQuery && payload != null)
			{
				// We say: You can drop on a GridView row only if at least one
				// of the dragged items is TeamMemberViewModel
				e.QueryResult = payload.DraggedItems.Any(item => item is TeamMemberViewModel);
				e.Handled = true;
			}
		}

		private static void OnGridViewScrollViewerDropQuery(object sender, DragDropQueryEventArgs e)
		{
			var scrollVeiwer = sender as GridViewScrollViewer;

			if (scrollVeiwer != null)
			{
				// TODO: We will be able to implement this once a few methods
				// of the GridViewScrollViewer are made public.
			}
		}

		private static void OnGridViewRowDropInfo(object sender, DragDropEventArgs e)
		{
			// #GridViewDropInfo Handle the drop event on a GridRow. We want
			// to show an appropriate message when a drop is possible and
			// handle the successful drop action by assigning an owner of the bug.
			var gridRow = sender as GridViewRow;
			var payload = e.Options.Payload as DragDropViewModel;
			var bug = gridRow.DataContext as BugViewModel;

			if (gridRow != null
				&& bug != null
				&& payload != null)
			{
				if (e.Options.Status == DragStatus.DropPossible)
				{
					var firstTeamMember = payload.DraggedItems.First(item => item is TeamMemberViewModel) as TeamMemberViewModel;
					var shortBugName = bug.Title;
					if (shortBugName.Length > 20)
					{
						shortBugName = "{0}...".F(shortBugName.Substring(0, 20));
					}
					payload.ApprovedDrops = payload.DraggedItems.Select(item => item == firstTeamMember);
					payload.Message = "Assign bug '{0}' to {1}".F(shortBugName, firstTeamMember.FullName);
					payload.IconPath = ApplicationIcons.AssignTo;
				}

				if (e.Options.Status == DragStatus.DropComplete)
				{
					var firstTeamMember = payload.DraggedItems.First(item => item is TeamMemberViewModel) as TeamMemberViewModel;

					bug.Owner = firstTeamMember;
				}
			}
		}

		private static void OnControlDropInfo(object sender, DragDropEventArgs e)
		{
			// #VisualFeedback: This event handler gives feedback to the user
			// that a drop is possible by setting the background of the
			// element. Please note that the event is not handled, because
			// there really is no concrete action that we do.
			var control = sender as Control;
			if (control != null && control == e.Options.Destination)
			{
				if (e.Options.Status == DragStatus.DropPossible)
				{
					control.Background = DataTemplatesProvider.Instance.DropPossibleBackground;
				}
				else if (e.Options.Status == DragStatus.DropImpossible
					|| e.Options.Status == DragStatus.DropComplete
					|| e.Options.Status == DragStatus.DropCancel)
				{
					control.ClearValue(Control.BackgroundProperty);
				}
			}
		}
	}
}
