﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using BugTrackerDragDrop.ViewModels;
using Telerik.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.DragDrop;

namespace BugTrackerDragDrop.Views
{
	public partial class SelectedBugsView : UserControl
	{
		public SelectedBugsView()
		{
			InitializeComponent();

			// #PropertiesInCode : Properties can also be set in code.
			RadDragAndDropManager.SetAllowDrop(this, true);

			this.AddHandler(RadDragAndDropManager.DropQueryEvent,
				new EventHandler<DragDropQueryEventArgs>(OnSelectedBugsDropQuery));

			this.AddHandler(RadDragAndDropManager.DropInfoEvent,
							new EventHandler<DragDropEventArgs>(OnSelectedBugsDropInfo));
		}

		private void OnSelectedBugsDropQuery(object sender, DragDropQueryEventArgs e)
		{
			var payload = e.Options.Payload as DragDropViewModel;

			if (e.Options.Status == DragStatus.DropDestinationQuery && payload != null)
			{
				e.QueryResult = payload.DraggedItems.Any(item => item is BugViewModel);
				e.Handled = true;
			}
		}

		private void OnSelectedBugsDropInfo(object sender, DragDropEventArgs e)
		{
			var payload = e.Options.Payload as DragDropViewModel;

			if (e.Options.Status == DragStatus.DropPossible && payload != null)
			{
				payload.ApprovedDrops = payload.DraggedItems.Cast<object>().Select(item => item is BugViewModel);
				payload.IconPath = ApplicationIcons.OpenItem;
				payload.Message = "Open Item";

				e.Handled = true;
			}

			if (e.Options.Status == DragStatus.DropComplete && payload != null)
			{
				foreach (var item in payload.DraggedItems.Where(item => item is BugViewModel).Cast<BugViewModel>())
				{
					item.IsSelected = true;
					tabControl.SelectedItem = item;
				}
			}
		}

        private void RadTabControl_SelectionChanged(object sender, RoutedEventArgs e)
        {
            var tabControl = sender as RadTabControl;
            if (tabControl.SelectedIndex == -1 && tabControl.Items.Count > 0)
            {
                tabControl.SelectedIndex = 0;
            }
        }
	}
}
