﻿using System;
using System.Linq;
using System.Windows.Controls;
using BugTrackerDragDrop.ViewModels;
using Telerik.Windows;
using Telerik.Windows.Controls.DragDrop;

namespace BugTrackerDragDrop.Views
{
	public partial class TeamCompareView : UserControl
	{
		public TeamCompareView()
		{
			InitializeComponent();

			this.AddHandler(RadDragAndDropManager.DropQueryEvent,
				new EventHandler<DragDropQueryEventArgs>(OnListBoxDropQuery));

			this.AddHandler(RadDragAndDropManager.DropInfoEvent,
							new EventHandler<DragDropEventArgs>(OnListBoxDrop));

		}

		private static void OnListBoxDropQuery(object sender, DragDropQueryEventArgs e)
		{
			var destination = e.Options.Destination as System.Windows.Controls.ListBox;
			var payload = e.Options.Payload as DragDropViewModel;

			if (e.Options.Status == DragStatus.DropDestinationQuery && payload != null && destination != null)
			{
				e.QueryResult = payload.DraggedItems.Any(item => item is BugViewModel);
				e.Handled = true;
			}
		}

		private static void OnListBoxDrop(object sender, DragDropEventArgs e)
		{
			var destination = e.Options.Destination as System.Windows.Controls.ListBox;
			var destinationTeamMember = destination.DataContext as TeamMemberViewModel;
			var payload = e.Options.Payload as DragDropViewModel;

			if (e.Options.Status == DragStatus.DropPossible && payload != null && destination != null && destinationTeamMember != null)
			{
				payload.ApprovedDrops = payload.DraggedItems.Cast<object>().Select(item => item is BugViewModel).ToList();
				payload.IconPath = ApplicationIcons.AssignTo;
				payload.Message = "Assign To {0}".F(destinationTeamMember.FullName);

				e.Handled = true;
			}

			if (e.Options.Status == DragStatus.DropComplete && payload != null && destination != null && destinationTeamMember != null)
			{
				foreach (var item in payload.DraggedItems.Where(item => item is BugViewModel).Cast<BugViewModel>())
				{
					item.Owner = destinationTeamMember;
				}
			}
		}
	}
}
