﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using BugTrackerDragDrop.ViewModels;
using Telerik.Windows.Controls;
using Telerik.Windows;
using Telerik.Windows.Controls.DragDrop;
using System.Diagnostics;

namespace BugTrackerDragDrop.Views
{
	public partial class TeamExplorerView : UserControl
	{
		public TeamExplorerView()
		{
			InitializeComponent();


            this.AddHandler(RadDragAndDropManager.DropQueryEvent,
                new EventHandler<DragDropQueryEventArgs>(OnTreeViewItemDropQuery));

            this.AddHandler(RadDragAndDropManager.DropInfoEvent,
                            new EventHandler<DragDropEventArgs>(OnTreeViewItemDrop));

            this.AddHandler(RadDragAndDropManager.DropQueryEvent,
                new EventHandler<DragDropQueryEventArgs>(OnTreeViewDropQuery));

            this.AddHandler(RadDragAndDropManager.DropInfoEvent,
                            new EventHandler<DragDropEventArgs>(OnTreeViewDrop));
		}

        private static void OnTreeViewItemDropQuery(object sender, DragDropQueryEventArgs e)
        {
            var destinationTeam = e.Options.Destination.DataContext as TeamViewModel;
			var destinationTeamMemeber = e.Options.Destination.DataContext as TeamMemberViewModel;

            var payload = e.Options.Payload as DragDropViewModel;

            if (e.Options.Status == DragStatus.DropDestinationQuery && payload != null)
            {
				if (destinationTeam != null)
				{
					e.QueryResult = payload.DraggedItems.Any(item => item is TeamMemberViewModel);
					e.Handled = true;
				}
				else if (destinationTeamMemeber != null)
				{
					e.QueryResult = payload.DraggedItems.Any(item => item is BugViewModel);
					e.Handled = true;
				}
            }
        }

        private static void OnTreeViewDropQuery(object sender, DragDropQueryEventArgs e)
        {
            var destination = e.Options.Destination as RadTreeView;
            var payload = e.Options.Payload as DragDropViewModel;

            if (e.Options.Status == DragStatus.DropDestinationQuery && payload != null && destination != null)
            {
                e.QueryResult = payload.DraggedItems.Any(item => item is TeamMemberViewModel);
                e.Handled = true;
            }
        }

        private static void OnTreeViewItemDrop(object sender, DragDropEventArgs e)
        {
            var destinationTeam = e.Options.Destination.DataContext as TeamViewModel;
			var destinationTeamMemeber = e.Options.Destination.DataContext as TeamMemberViewModel;
            var payload = e.Options.Payload as DragDropViewModel;

            if (e.Options.Status == DragStatus.DropPossible && payload != null)
            {
				if (destinationTeam != null)
				{
					payload.ApprovedDrops = payload.DraggedItems.Cast<object>().Select(item => item is TeamMemberViewModel).ToList();
					payload.IconPath = ApplicationIcons.AddToTeam;
					payload.Message = "Move to the {0}".F(destinationTeam.TeamName);
					e.Handled = true;
				}
				else if (destinationTeamMemeber != null)
				{
					payload.ApprovedDrops = payload.DraggedItems.Cast<object>().Select(item => item is BugViewModel).ToList();
					payload.IconPath = ApplicationIcons.AssignTo;
					payload.Message = "Assign To {0}".F(destinationTeamMemeber.FullName);
					e.Handled = true;
				}
            }

            if (e.Options.Status == DragStatus.DropComplete && payload != null)
            {
				if (destinationTeam != null)
				{
					foreach (var item in payload.DraggedItems.Where(item => item is TeamMemberViewModel).Cast<TeamMemberViewModel>())
					{
						item.Team = destinationTeam;
					}
				}
				if (destinationTeamMemeber != null)
				{
					foreach (var item in payload.DraggedItems.Where(item => item is BugViewModel).Cast<BugViewModel>())
					{
						item.Owner = destinationTeamMemeber;
					}
				}
            }
        }

        private static void OnTreeViewDrop(object sender, DragDropEventArgs e)
        {
            var destination = e.Options.Destination as RadTreeView;
            var payload = e.Options.Payload as DragDropViewModel;

            if (e.Options.Status == DragStatus.DropPossible && payload != null && destination != null)
            {
                payload.ApprovedDrops = payload.DraggedItems.Cast<object>().Select(item => item is TeamMemberViewModel).ToList();
                payload.IconPath = ApplicationIcons.AddToTeam;
                payload.Message = "Remove From All Teams";

                e.Handled = true;
            }

            if (e.Options.Status == DragStatus.DropComplete && payload != null && destination != null)
            {
                foreach (var item in payload.DraggedItems.Where(item => item is TeamMemberViewModel).Cast<TeamMemberViewModel>())
                {
                    item.Team = item.Team = null;
                }
            }
        }
	}
}
